#!/usr/local/bin/perl

package h2;

sub main::header {
    $site_type = shift(@_);
    $site   = uc(shift(@_));
    $year   = shift(@_);
    $month  = shift(@_);
    $days   = shift(@_);
    @dates_to_proc = @_;
    $start  = sprintf "%4d%02d%02d",$year,$month,"1";
    $end    = sprintf "%4d%02d%02d",$year,$month,$days;

	###  Read Table Definitions.
	define_tables();

	###  Read tables
	$ti = read_table('./Tables/instrument.tbl',  $ri, \%di, 'WRMCID');
	$tc = read_table('./Tables/calibration.tbl', $rc, \%dc, 'WRMCID');
	$ta = read_table('./Tables/action.tbl',      $ra, \@da);
	$td = read_table('./Tables/deployment.tbl',  $rd, \%dd, 'WRMCID');

	# Filter Action table, leaving only rows for $site.
	@$ta = grep { uc($_->{SITE}) eq $site } @$ta;

	### Create Indicies for some tables
	$iti = make_index($ti, 'WRMCID');	# Index Table Instrument  ($ti)
	$itd = make_index($td, 'WRMCID');	# Index Table Deployment  ($td)
	$itc = make_index($tc, 'WRMCID');	# Index Table Calibration ($tc)

	#>>>  show_table($ti, 'Instrument');
	#>>>  show_table($tc, 'Calibration');
	#>>>  show_table($ta, 'Action');
	#>>>  show_table($td, 'Deployment');
	#>>>  show_index($iti, "Index on Instrument");
	#>>>  show_index($itc, "Index on Calibration");

	do_header($ta, $ti, $tc, $start, $end);
}

sub do_header {
	my ($ta, $ti, $tc, $start, $end) = @_;
	my ($now, @z, @current_inst, %inst_list, $id, %before, %qmeas, $depl);
	my ($d, $h, $m);

	###
	###  Prepare $start and $end
	###
	# ASCII range check for dates
	$start .= '0000';
	$end   .= '2359';
	@z = localtime();
	$now = sprintf("%04d%02d%02d0000", $z[5]+1900, $z[4]+1, $z[3]);

	###
	### Check for invalid date input
	###	DANGER, MAGIC NUMBER below
	if (  $start lt '199301010000' ||  $start gt $now ||
		$end lt '199301010000' ||  $end lt $start ) {
#				||  $end gt $now  ||  $end lt $start ) {
		warn "Error:  Dates out of range.  Is format YYYYMMDD ?\n";
		#>>>   warn "  start=$start, end=$end, now=$now\n";
		return;
	}

	#
	#  Scan through action table, noting Up/Dn states in current time range
	#	@current_inst  will hold all rows from action tbl in current
	#							time range.
	#	%before        will hold single last row BEFORE current time
	#				range for each unique instrument
	#				(indexed by instrument ID).
	#
	undef %before, @current_inst;		# These are now clean.
	foreach $r (@$ta) {
		$id = $r->{WRMCID};
		# Is Action in time range?
		if (($r->{TIME} ge $start)  &&  ($r->{TIME} le $end)) {
			#  Accumulate all rows in range
			# warn "PUSHING $id, time=$r->{TIME}\n";
			push(@current_inst, $r);

		} elsif (($r->{TIME} lt $start) &&
			 ($r->{TIME} gt $before{$id}->{TIME})) {
			$before{$id} = $r;   # Note last row before time range
		}
	}

	#
	#  Prepend last record before timeframe for each instrument
	#	@current_inst then will have all required header instrument
	#	instances, not including instruments which were down before
	#	the start of the current period.
	#
	foreach $k (keys(%before)) {
		#  Add record ONLY IF not out of service
		if ($before{$k}->{ACTION} ne 'down') {
			#>>> warn "UNSHIFTING $k\n";
			unshift(@current_inst, $before{$k});
		}
	}

	#
	#  Compute PARAMMEASURED for each of the @current_inst
	#	-- Scan for last deployment before instrument's action.

	foreach $r (@current_inst) {
		my $deploy_time = '';
		foreach $d (@$td) {
			next  if ($d->{WRMCID} ne $r->{WRMCID});
			next  if ($d->{TIME} gt $r->{TIME});
			next  if ($d->{TIME} lt $deploy_time);

			$r->{MVKEY} = $mvkey{$d->{PARAMMEASURED}};
			$deploy_time = $d->{TIME};
# print STDOUT ">>>> NOTE $r->{MVKEY} at $d->{TIME}\n";
		}
	}

	# Compute List of MVKEYS (BSRN Tbl. 4.13) for U0001 section
	# ForEach instrument in action during current period,
	#	determine and note the MVkey for its ParamMeasured.
	foreach $r (@current_inst) {
	  #>>>warn "$r->{WRMCID} meas. $itd->{$r->{WRMCID}}{PARAMMEASURED}\n";
		$id =  $r->{WRMCID};
		$depl = most_recent($td, 'TIME', $r->{TIME}, 'WRMCID', $id);

		# Create an element with QtyMeasCode as key foreach instrument.
		$qmeas{$mvkey{$depl->{PARAMMEASURED}}} = 1;
	}

	# Add MVKEYS of non-radiation instruments that will not be included
	#    in section U0008 (see %mvkeys_notin0008 below)
	# 
	foreach $k (@mvkeys_notin0008) {
		next  if ! exists($mvkey{$k});	# Skip if example QtyMeasCode
		$qmeas{$mvkey{$k}} = 1;
	}

	# Sort numerically ({$a<=>$b}) the list of UNIQUE QtyMeasCodes
	# @mvkeylist is used directly by header1().
	@mvkeylist = sort {$a <=> $b} keys(%qmeas);

#>>>
#	foreach $k (sort {$a <=> $b} keys(%qmeas)) {
#		warn "ID $k found $qmeas{$k}\n";
#	}

	###
	### Fill @mvkeylist to even 8-item boundary with '-1's.
	###	$n is the number of '-1's to add.
	###
	#>>> $n = (8 - scalar(@mvkeylist) % 8) % 8;
	#>>> alternative:  push(@mvkeylist, (-1) x $n);	# using repeat operator.

	$n = (8 - scalar(@mvkeylist)) % 8;

	for ($i=0; $i < $n; $i++) {
		push(@mvkeylist, -1);
	}

	###
	###  Header 1 Output (U0001 .. U0007)
	###

	main::header1($site_type, lc($site), $year, $month, @dates_to_proc);

	###
	###  U0008 Output
	###

	#  Sort @current_inst primarily by the order given by %bytype
	#	(by the instrument's param measured), secondarily by time.
	@current_inst = sort {
#			$bytype{uc($itd->{$a->{WRMCID}}->{PARAMMEASURED})} <=>
#			$bytype{uc($itd->{$b->{WRMCID}}->{PARAMMEASURED})}
			$a->{MVKEY} <=> $b->{MVKEY}
			|| $a->{TIME} cmp $b->{TIME}
		}  @current_inst;

#### DEBUGGING ####
# print STDOUT "------------------------------\n";
# foreach $r (@current_inst) {
# 	print STDOUT ">>>> INST: $r->{WRMCID} measures $r->{PARAMMEASURED}\n";
# }
# show_table($ti);
#### DEBUGGING ####


	###
	### Foreach instrument in @current_inst, output a U0008 record.
	###
	foreach $r (@current_inst) {
		h2record0008($r);
	}

	###
	###  U0009 Output
	###
	print "*U0009\n";

	###
	### Foreach instrument in @current_inst, output a U0009 record.
	###
	foreach $r (@current_inst) {
#print ">>>> INST: $r->{WRMCID} measures $itd->{$r->{WRMCID}}->{PARAMMEASURED} ($bytype{uc($itd->{$r->{WRMCID}}->{PARAMMEASURED})})\n";
		h2record0009($r);
	}
}

sub h2record0008 {
	my ($r) = @_;
	my ($id, $inst, $depl, $cal, $pm, $nowtime);

	###
	### 0008 line 1
	###
#	warn ">>> Instrument $id: time=$r->{TIME}\n"  if $debug;
	if ($r->{TIME} lt $start) {
		printf(" -1 -1 -1 %s\n",
				lc($r->{ACTION}) eq 'up' ? 'Y' : 'N');
	} else {
		($d, $m, $h) = $r->{TIME} =~ m/\d{6}(\d\d)(\d\d)(\d\d)/;
		printf(" %2d %2d %2d %s\n", $d, $m, $h,
				lc($r->{ACTION}) eq 'up' ? 'Y' : 'N');
	}

	###
	### General calculations determining this instrument's
	###	$id (WRMCID), $inst, $cal, and $depl.
	###
	$id    = $r->{WRMCID};
	$inst  = $iti->{$id};
	# Last cal record for this instrument before this h2-action timestamp
	$nowtime = substr($r->{TIME}, 0, 8);
	$cal   = most_recent($tc, 'ENDCAL1', $nowtime, 'WRMCID', $id);
	$depl  = most_recent($td, 'TIME', $r->{TIME}, 'WRMCID', $id);

	if ( ! defined($inst) || ref($inst) ne 'HASH') {
		warn "Error:  No Instrument data for WRMCID $id";
		return;
	}
	if ( ! defined($depl) || ref($depl) ne 'HASH') {
		warn "Error:  No Deployment data for WRMCID $id";
		return;
	}
	if ( ! defined($cal)  || ref($cal) ne 'HASH') {
		warn "Error:  No Calibration data for WRMCID $id";
		return;
	}

	###
	### 0008 line 2, 3
	###
	printf("%-30.30s %-15.15s %-18.18s %-8.8s %d\n",
			$inst->{MANUFACTURER},
			$inst->{MODEL},
			$inst->{SERIALNO},
			h2date($inst->{PURCHASEDATE}), $id);
	printf("%-80.80s\n", $depl->{REMARKS});

	###
	###  Calibration Information, (0008 line 4 .. 10)
	###
	# FUTURE:  Support multiple calibration records in an h2-record
	$pm = uc($depl->{PARAMMEASURED});
	if ( ! exists($parammeasured{$pm})) {
		warn "Error:  No such parameter '$pm'\n";
		return;
	}

	printf(" %2d %2d %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %2d %2d\n",
		@{$parammeasured{$pm}});			# Line 4
	printf("%-30.30s %-40.40s\n",
		$cal->{CALLOCATION}, $cal->{CALPERSON});	# Line 5
	printf("%-8.8s %-8.8s %2d %12.4f %12.4f\n",
		h2date($cal->{STARTCAL1}),
		h2date($cal->{ENDCAL1}),
		h2num($cal->{COMPNUMBER1}),
		h2num($cal->{COEFFICIENT1}),
		h2num($cal->{STDERROR1}));			# Line 6
	printf("%-8.8s %-8.8s %2d %12.4f %12.4f\n",
		h2date($cal->{STARTCAL2}),
		h2date($cal->{ENDCAL2}),
		h2num($cal->{COMPNUMBER2}),
		h2num($cal->{COEFFICIENT2}),
		h2num($cal->{STDERROR2}));			# Line 7
	printf("%-8.8s %-8.8s %2d %12.4f %12.4f\n",
		h2date($cal->{STARTCAL3}),
		h2date($cal->{ENDCAL3}),
		h2num($cal->{COMPNUMBER3}),
		h2num($cal->{COEFFICIENT3}),
		h2num($cal->{STDERROR3}));			# Line 8
	printf("%-80.80s\n", $cal->{REMARKSUNITS} || 'XXX');	# Line 9
	printf("%-80.80s\n", $cal->{REMARKSMISC} || 'XXX');	# Line 10
}

sub h2record0009 {
	my ($r) = @_;
	my ($id, $inst, $depl, $pm);

	###
	###  0009 Day/Month/Hour
	###
#	warn ">>> Instrument $id: time=$r->{TIME}\n"  if $debug;
	if ($r->{TIME} lt $start) {
		print ' -1 -1 -1';
	} else {
		($d, $m, $h) = $r->{TIME} =~ m/\d{6}(\d\d)(\d\d)(\d\d)/;
		printf(" %2d %2d %2d", $d, $m, $h);
	}

	$id    = $r->{WRMCID};
	$depl  = most_recent($td, 'TIME', $r->{TIME}, 'WRMCID', $id);

	if ( ! defined($depl) || ref($depl) ne 'HASH') {
		warn "Error:  No Deployment data for WRMCID $id";
		return;
	}

	###
	###  0009 QtyMeasCode, WRMCID, BandNumber
	###
	printf(" %9d %5d %2d\n", $mvkey{$depl->{PARAMMEASURED}}, $id,
			$band0009{$depl->{PARAMMEASURED}});

}

#
#  h2 record date:  either MM/DD/YY or 'XXX' if undefined
#
sub h2date {
	my ($d) = @_;

	if ($d) {
		return(sprintf("%02d/%02d/%02d",
				($d =~ m/\d\d(\d\d)(\d\d)(\d\d)/)[1,2,0]));
	} else {
		return ('XXX');
	}
}

#
#  h2 record number:  either MM/DD/YY or 'XXX' if undefined
#
sub h2num {
	my ($n) = @_;

	return ($n eq '' ? -1 : $n);
}


#  read_table() --  Read table contents from file
#
#  read_table( <filename>, <regex>, <description>, <delim> )
#	Parameters:
#		<filename>	String filename of file containing table
#		<regex> 	Regular expression to use in parsing lines
#						from table file.
#		<description>	Reference to:
#					HASH  - describing paragraph format tbl.
#						Hash Keys are field names,
#						Values are field types.
#					ARRAY - describing line format tbl
#						Values are field names
#		<delim>		Name of field which identifies start of record
#
#  Use: 
#    Read Table in Paragraph form (1 rec/Paragraph)
# 	$t = read_table("filename", 'regex', \%fields)
#    Read Table in Column form (1 rec/line)
# 	$t = read_table("filename", 'regex', \@fields)
#    Returns reference to 2D table
sub read_table {
	my ($filename, $regex, $r, $key) = @_;
	my ($t, $row, $line);

	open(T, $filename) || die "Can't open table input file $filename: $!\n";

	if (ref($r) eq 'ARRAY') {	# Read Line format table
		while (<T>) {
			next  if /^\s*$/;		# Skip blank lines
			next  if /^\s*#/;		# Skip comment lines
			s/#.*$//;			# Strip tag comment

#>>>			push(@$t, [ /$regex/o ]);	# Parse row, push

###  This is an alternative to the single line below
#			$_ =~ /^\s*(\w+)\s+(down|up)\s+(\d{12})\s+(\w+)/
#			$row = { };
#			${$row}{SITE}   = $1;
#			${$row}{ACTION} = $2;
#			${$row}{TIME}   = $3;
#			${$row}{WRMCID} = $4;

			##
			## Parse table row, implicitly create hash ref $row
			##   $row points to a "Row Object"
			##
			@{$row}{@$r} = ( m/$regex/o );	# Parse row, create Arr.

			##
			## Append row to table, implicitly create array ref $t
			##   $t points to a "Table Object"
			##
			push(@$t, $row);		# Push row, $t is table
			undef $row;
		}

	} elsif (ref($r) eq 'HASH') {	# Read Paragraph format table
		while (<T>) {
			$line++;
			next  if /^\s*$/;		# Skip blank lines
			next  if /^\s*#/;		# Skip comment lines

			if ( ! /$regex/o) {
				warn "($filename:$line):  bad format\n";
				next;
			}
			($k, $v) = (uc($1), $2);

			if ( ! exists(${$r}{$k})) {
				warn "($filename:$line): Bad fieldname '$k'\n";
				next;
			}

			# Store old row in table, start new row
			if ($k eq $key  && defined($row)) {
				push(@$t, $row);
				undef $row;
			}

			$row->{$k} = $v;
		}
		push(@$t, $row);		# Store final row
	} else {
		die "Error:  incorrect use of read_table\n";
	}

	close(T);
	return ($t);
}

###
###  most_recent()  -- Return ref to table row most recent to 'nowtime',
###			with restriction that 'selectfield'='value'.
###			Time specified in table by 'timefieldname'.
###	 $mr = most_recent(table, timefieldname, nowtime, selectfield, value);
### e.g. $mr = most_recent($tc, 'ENDCAL1', $end, 'WRMCID', $id);
#	$cal   = most_recent($tc, 'ENDCAL1', $r->{TIME}, 'WRMCID', $id);
sub most_recent {
	my ($t, $tf, $since, $sf, $sv) = @_;
	my ($mr, $row);

	undef $mr;
	foreach $row (@$t) {
		#>>> print "\t$row->{$tf} cmp $since\n";
		if ($row->{$sf} eq $sv  &&  $row->{$tf} le $since) {
			#>>> print STDERR "$row->{$tf} LESS THAN $since\n";
			if (defined($mr)) {
				if ($mr->{$tf} lt $row->{$tf}) {
					$mr = $row;
				}
			} else {
				$mr = $row;
			}
		}
	}
#XXX
	#>>> print STDERR ">>> lastcal $sv since $since is $mr->{ENDCAL1}\n";
	#  Return index hash ref
	return ($mr);
}

###
###  
###
sub filter_table {
	my ($t, $colname, $val) = @_;
	my ($r, $new);

	@$new = grep { uc($_->{$colname}) eq uc($val) } @$t;
	

	return ($new);
}

sub make_index {
	my ($t, $key) = @_;
	my ($row, $ix);

	foreach $row (@$t) {
		$ix->{$row->{$key}} = $row;
	}
	#  Return index hash ref
	return ($ix);
}

###
###  show_table -- Display table contents (mainly for debugging)
###
sub show_table {
	my ($t, $table_name) = @_;
	my ($row, $key);

	print STDERR "===== $table_name ==================================\n";
	foreach $row (@$t) {
		print STDERR "--------------------------\n";
		foreach $key (keys(%$row)) {
			printf(STDERR "%-16s : [%s]\n", $key, $row->{$key});
		}
	}
}

sub show_index {
	my ($t, $table_name) = @_;
	my ($row, $key);

	print STDERR "===== INDEX: $table_name ==================================\n";
	foreach $key (sort(keys(%$t))) {
		$row = $t->{$key};
		printf(STDERR "%-16s: %s of %d\n", $key, ref($row), scalar keys %$row);
#		my @k = %$row;
#		my @k = map( ($_, '['.length($_).']'), sort keys %$row);
#		my @k = map "[$_]", sort keys %$row;
		my @k = sort keys %$row;
		print STDERR "\t\t@k\n";
	}
}

sub define_tables {
	##
	## Instrument Table field names and types:
	##
	%di = (
		WRMCID		=> 'n',
		MANUFACTURER	=> 's',
		MODEL		=> 's',
		SERIALNO	=> 's',
		PURCHASEDATE	=> 'd',
		TIMESTAMP	=> 't'
	);
	$ri = '^(\w+)\s*:\s*(.*?)\s*$';			# RegEx for Instr. Tbl.

	##
	## Calibration Table field names:
	##
	%dc = (
		WRMCID		=> 'n',
		TIMESTAMP	=> 't',
		CALLOCATION	=> 's',
		CALPERSON	=> 's',
		STARTCAL1	=> 'd',
		ENDCAL1		=> 'd',
		COMPNUMBER1	=> 'n',
		COEFFICIENT1	=> 'n',
		STDERROR1	=> 'n',
		STARTCAL2	=> 'd',
		ENDCAL2		=> 'd',
		COMPNUMBER2	=> 'n',
		COEFFICIENT2	=> 'n',
		STDERROR2	=> 'n',
		STARTCAL3	=> 'd',
		ENDCAL3		=> 'd',
		COMPNUMBER3	=> 'n',
		COEFFICIENT3	=> 'n',
		STDERROR3	=> 'n',
		REMARKSUNITS	=> 's',
		REMARKSMISC	=> 's',
	);
	$rc = '^(\w+)\s*:\s*(.*?)\s*$';			# RegEx for Cal. Tbl.

	##
	## Action Table field names:
	##
	@da = qw/SITE ACTION TIME WRMCID/;
	$ra = '^\s*(\w+)\s+(down|up)\s+(\d{12})\s+(\w+)';  # RegEx for Act. Tbl.

	##
	##  Deployment Table field names
	##
	%dd = (
		WRMCID		=> 'n',
		TIME		=> 't',
		PARAMMEASURED	=> 's',
		REMARKS		=> 's',
	);
	$rd = '^(\w+)\s*:\s*(.*?)\s*$';			# RegEx for Dplym. Tbl.

	# Values in record 0008 line 4 for each type (BSRN tech plan Tbl. 4.15)
	%parammeasured = (
		GLOBAL_SW	=> [ (-1) x 10 ],
		DIRECT_SW	=> [ (-1) x 8, 90, -1 ],
		DIFFUSE_SW	=> [ (-1) x 10 ],
		INCOMING_LW	=> [ 3, 6, (-1) x 8 ],
		INCOMING_LWS	=> [ 3, 7, (-1) x 8 ],
                UVB_GLOBAL      => [ (-1) x 10 ],
		REFLECT_SW	=> [ (-1) x 10 ],
		REFLECT_SW10	=> [ (-1) x 10 ],
		OUTGOING_LW	=> [ 3, 3, (-1) x 8 ],
		OUTGOING_LW10	=> [ 3, 3, (-1) x 8 ],
	);

	# mvkey (param meas) from 0009 in record. See BSRN tech plan Tbl. 4.13.
	# Also used to generate param_measured list in 0001 record line 2
	%mvkey = (
		GLOBAL_SW	=> 2,		# Alias for DSGL2
		DIRECT_SW	=> 3, 		# Alias for DSDIR
		DIFFUSE_SW	=> 4, 		# Alias for DSDFS
		INCOMING_LW	=> 5,		# Alias for DL
		INCOMING_LWS	=> 5,		# Alias for DL
		DSGL2		=> 2,
		DSDIR		=> 3, 
		DSDFS		=> 4, 
		DL		=> 5,
		DLAT		=> 21,
		DLRH		=> 22,
		DLPRES		=> 23,
                UVB_GLOBAL      => 123,
		REFLECT_SW	=> 131,		# Alias for USR
		OUTGOING_LW	=> 132,		# Alias for UL
		REFLECT_SW10	=> 131001000,	# Alias for USRH1000
		OUTGOING_LW10	=> 132001000,	# Alias for ULH1000
		USR		=> 131,
		UL		=> 132,
		NETR		=> 141,
		DSGL2H700	=> 2000700,
		USRH700		=> 131000700,
		ULH700		=> 132000700,
		DLH700		=> 5000700,
		DLATH700	=> 21000700,
		DLRHH700	=> 22000700,
		DSGL2H1000	=> 2001000,
		USRH1000	=> 131001000,
		ULH1000		=> 132001000,
		DLH1000		=> 5001000,
		DLATH1000	=> 21001000,
		DLRHH1000	=> 22001000,
		USRH3000	=> 131003000,
		ULH3000		=> 132003000,
		DLH3000		=> 5003000,
		DLATH3000	=> 21003000,
		DLRHH3000	=> 22003000,
		DSSP4		=> 104,
		DSSP12		=> 112,
		SSCLTI		=> 301,
		SSCLBI		=> 302,
		SSCLLW		=> 303,

		PAR		=> 123451,
		'WIND MONITOR'	=> 123452,
		BAROMETER	=> 23,		# Alias DLPRES

	);

	# MVKEYS of non-radiation instruments that will not be included
	#    in section U0008 (see %mvkeys_notin0008 below) BY site
	%mvkeys_notin0008 = (
#                surfrad	=> [ qw/DLATH1000 DLRHH1000 DLPRES/ ],
#		twp	=> [ qw/DLAT DLRH DLPRES/ ],
#		TWP	=> [ qw/DLAT DLRH DLPRES/ ],
#		sgp	=> [ qw/DLAT DLRH DLPRES/ ],
#		bsrn	=> [ ],
		DRA	=> [ qw/DLATH1000 DLRHH1000 DLPRES/ ],
		BOS	=> [ qw/DLATH1000 DLRHH1000 DLPRES/ ],
		FPE	=> [ qw/DLATH1000 DLRHH1000 DLPRES/ ],
		BON	=> [ qw/DLATH1000 DLRHH1000 DLPRES/ ],
		GCR	=> [ qw/DLATH1000 DLRHH1000 DLPRES/ ],
		PSU	=> [ qw/DLATH1000 DLRHH1000 DLPRES/ ],
		SXF	=> [ qw/DLATH1000 DLRHH1000 DLPRES/ ],
		MAN	=> [ qw/DLAT DLRH DLPRES/ ],
		NAU	=> [ qw/DLAT DLRH DLPRES/ ],
		E13	=> [ qw/DLAT DLRH DLPRES/ ],
		DAR	=> [ qw/DLAT DLRH DLPRES/ ],
		BIL	=> [ qw// ],
		BRS	=> [ qw// ],
	);
#	@mvkeys_notin0008 = @{ $mvkeys_notin0008{ $site_type } };
	@mvkeys_notin0008 = @{ $mvkeys_notin0008{ $site } };

	# Band number for spectral instr. for record 0009. (-1 => Broadband)
	%band0009 = (
		GLOBAL_SW	=> -1,
		DIRECT_SW	=> -1, 
		DIFFUSE_SW	=> -1, 
		INCOMING_LW	=> -1,
		INCOMING_LWS	=> -1,
                UVB_GLOBAL      => -1,
		REFLECT_SW	=> -1,
		REFLECT_SW10	=> -1,
		OUTGOING_LW	=> -1,
		OUTGOING_LW10	=> -1,
	);

	# These numbers specify the SORT ORDER in record 0008 and 0009
	%bytype = (
		GLOBAL_SW	=> 1,
		DIRECT_SW	=> 2,
		DIFFUSE_SW	=> 3,
		INCOMING_LW	=> 4,
		INCOMING_LWS	=> 5,
                UVB_GLOBAL      => 6,
		REFLECT_SW	=> 7,
		REFLECT_SW10	=> 8,
		OUTGOING_LW	=> 9,
		OUTGOING_LW10	=> 10,

	);
}

1;
__END__

# Instrument Information File
WRMCID		: 28002
Manufacturer	: Eppley
Type		: PSP
SerialNo	: 30720E6
PurchaseDate	:
Comment		: Located ~200 yards from downpointing instruments
TimeStamp	: 199810070915

# Calibration Information File
WRMCID          : 28024
CalLocation     : ARM/CART SITE CAL TRAILER
CalPerson       : CRAIG WEBB
StartCal1       : 08/12/99
EndCal1         : 08/12/99
CompNumber1     : 
Coefficient1    : 9.033
StdError1       : 
RemarksUnits    : UNITS OF THE CALIBRATION COEFFICIENT ARE [mV/(W/m^2)]
RemarksMisc     : THIS IS THE PSP MEASURING GLOBAL (TOTAL) SOLAR IRRADIANCE


#Action	YYYYMMDDHHmm	id	# Initials	Comment
up	199707311520	28024	# PTK		Put into service this date
down	199804221045	28024	# GH		Swap:  replaced by 28017

%definition = (
   FieldName => [
		validating regex,
		validating subroutine,
		required or not,
		],
   ...
)
